Module map

Module map 

Structs

Map
This Map uses a similar strategy to BTreeMap to ensure cache efficient performance on modern hardware while still providing log(N) get, insert, and remove operations.
NodeHandle
An owned node, built by create or kept from a NodeRef. A map is assembled from handles with Map::from_root.
NodeRef
A borrowed node of a map’s tree, for a codec that must reproduce the tree’s sharing. Two views with the same identity are the same node; a NodeHandle from keep pins that identity for as long as it is held.
WeakMapRef
A weak reference to a map.

Constants

DEFAULT_SIZE

Type Aliases

MapL
Map using a larger chunk size, faster to search, slower to update
MapM
Map using the default chunk size, a good balance of update and search
MapS
Map using a smaller chunk size, faster to update, slower to search
WeakMapRefL
WeakMapRefM
WeakMapRefS