Struct immutable_chunkmap::set::Set[][src]

pub struct Set<K: Ord + Clone> { /* fields omitted */ }

This set uses a similar strategy to BTreeSet to ensure cache efficient performance on modern hardware while still providing log(N) get, insert, and remove operations.

Examples

use std::string::String;
use self::immutable_chunkmap::set::Set;

let m =
   Set::new()
   .insert(String::from("1")).0
   .insert(String::from("2")).0
   .insert(String::from("3")).0;

assert_eq!(m.contains("1"), true);
assert_eq!(m.contains("2"), true);
assert_eq!(m.contains("3"), true);
assert_eq!(m.contains("4"), false);

for k in &m { println!("{}", k) }

Methods

impl<K> Set<K> where
    K: Ord + Clone
[src]

Create a new empty set

This will insert many elements at once, and is potentially a lot faster than inserting one by one, especially if the data is sorted.

#Examples

 use self::immutable_chunkmap::set::Set;

 let mut v = vec![1, 10, -12, 44, 50];
 v.sort_unstable();

 let m = Set::new().insert_many(v.iter().map(|k| *k));

 for k in &v {
   assert_eq!(m.contains(k), true)
 }

Remove multiple elements in a single pass. Similar performance to insert_many.

This is just slightly wierd, however if you have a bunch of borrowed forms of members of the set, and you want to look at the real entries and possibly add/update/remove them, then this method is for you.

return a new set with k inserted into it. If k already exists in the old set return true, else false. If the element already exists in the set memory will not be allocated.

return true if the set contains k, else false. Runs in log(N) time and constant space. where N is the size of the set.

return a reference to an item in the set if any that is equal to the given value.

return a new set with k removed. Runs in log(N) time and log(N) space, where N is the size of the set

get the number of elements in the map O(1) time and space

Important traits for SetIter<'a, Q, K>

return an iterator over the subset of elements in the set that are within the specified range.

The returned iterator runs in O(log(N) + M) time, and constant space. N is the number of elements in the tree, and M is the number of elements you examine.

if lbound >= ubound the returned iterator will be empty